import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Button, notification, Modal } from 'antd'

import Api from '@/api'
import zhCN from '@/locales/zh-CN/main.js'
import enUS from '@/locales/en-US/main.js'
import MKEmitter from '@/utils/events.js'
import './index.scss'

const { confirm } = Modal

class NewPageButton extends Component {
  static propTpyes = {
    show: PropTypes.any,              // 按钮显示样式控制
    BID: PropTypes.string,            // 主表ID
    btn: PropTypes.object,            // 按钮
    selectedData: PropTypes.any,      // 子表中选择数据
    setting: PropTypes.any,           // 页面通用设置
    updateStatus: PropTypes.func,     // 按钮状态更新
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS,
    loading: false
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    const { show } = this.props

    if (show === 'actionList') {
      MKEmitter.addListener('triggerBtnId', this.actionTrigger)
    }
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('triggerBtnId', this.actionTrigger)
  }
  
  /**
   * @description 触发按钮操作
   */
  actionTrigger = (triggerId) => {
    const { setting, selectedData, btn } = this.props
    const { loading } = this.state
    
    if ((triggerId && btn.uuid !== triggerId) || loading) return
    
    let data = selectedData || []

    if (data.length !== 1) {
      // 需要选择单行时，校验数据
      notification.warning({
        top: 92,
        message: this.state.dict['main.action.confirm.selectSingleLine'],
        duration: 5
      })
      return
    } else if (!setting.primaryKey) {
      // 需要选择行时，校验是否设置主键
      notification.warning({
        top: 92,
        message: '未设置主键！',
        duration: 5
      })
      return
    }

    let primaryId = data[0][setting.primaryKey] || ''

    this.setState({loading: true})
    this.changeUser(primaryId)
  }

  /**
   * @description 切换用户
   */
  changeUser = (primaryId) => {
    const { setting, btn } = this.props
    let _this = this

    let param = {
      func: 'webapi_ChangeUser'
    }

    if (this.props.BID) {
      param.BID = this.props.BID
    }

    if (window.GLOB.mainSystemApi) {
      param.rduri = window.GLOB.mainSystemApi
    }

    param[setting.primaryKey] = primaryId

    confirm({
      title: this.state.dict['main.action.confirm.tip'],
      onOk() {
        return new Promise(resolve => {
          Api.genericInterface(param).then(res => {
            resolve()
            if (res.status) {
              sessionStorage.setItem('UserID', res.UserID)
              sessionStorage.setItem('LoginUID', res.LoginUID)
              sessionStorage.setItem('User_Name', res.UserName)
              sessionStorage.setItem('Full_Name', res.FullName)
              sessionStorage.setItem('avatar', res.icon || '')
              sessionStorage.setItem('dataM', res.dataM ? 'true' : '')
              sessionStorage.setItem('localDataM', res.dataM ? 'true' : '')
              sessionStorage.setItem('debug', res.debug || '')
              sessionStorage.setItem('role_id', res.role_id || '')
              sessionStorage.setItem('localRole_id', res.role_id || '')
              
              sessionStorage.removeItem('CloudAvatar')
              sessionStorage.removeItem('cloudDataM')
              sessionStorage.removeItem('CloudUserID')
              sessionStorage.removeItem('CloudUserName')
              sessionStorage.removeItem('CloudLoginUID')
              sessionStorage.removeItem('CloudFullName')
              
              window.location.reload()
            } else {
              notification.error({
                top: 92,
                message: res.message || res.ErrMesg,
                duration: btn.verify && btn.verify.ntime ? btn.verify.ntime : 10
              })
              _this.setState({loading: false})
            }
          }, () => {
            resolve()
            _this.setState({loading: false})
          })
        })
      },
      onCancel() {
        _this.setState({loading: false})
      }
    })
  }

  render() {
    const { btn, show } = this.props
    const { loading } = this.state

    if (show === 'actionList') {
      return (
        <Button
          icon={btn.icon}
          loading={loading}
          className={'mk-btn mk-' + btn.class}
          onClick={(e) => {e.stopPropagation(); this.actionTrigger()}}
        >{btn.label}</Button>
      )
    } else { // icon、text、 all 卡片
      return (
        <Button
          type="link"
          loading={loading}
          icon={show === 'text' ? '' : (btn.icon || '')}
          onClick={(e) => {e.stopPropagation(); this.actionTrigger()}}
        >{show === 'icon' && btn.icon ? '' : btn.label}</Button>
      )
    }
  }
}

export default NewPageButton